
/* Copyright (C) 1993 Gary A. Moyer
     Written by Gary A. Moyer (moyerg@cs.pdx.edu)

This file is part of the Secure Filesystem (SFS).

The SFS is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

The SFS is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with the SFS; see the file COPYING.  If not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */


/*
 * $Author: gary $
 * $Log: acl.c,v $
 * Revision 1.5  1993/08/16  10:44:38  gary
 * fixing location of u,g, and o object entries within the
 * list itself.
 *
 * Revision 1.4  1993/08/09  22:44:17  gary
 * adding name recognition to acl_from_text() for
 * group and user entries
 *
 * Revision 1.2  1993/07/29  22:18:39  gary
 * moving dynamic allocation of ACL from acl_get_file() into
 * acl_copy_ext().
 *
 * Revision 1.1  1993/06/29  20:56:24  gary
 * Initial revision
 *
 */

#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <limits.h>
#include <grp.h>
#include <pwd.h>
#include "acl.h"

#define ILLEGAL_PERMS(x) (x & (255 ^ (ACL_EXECUTE | ACL_READ | ACL_WRITE)))
#define TAGTYPE(x) (x & 2 ? 'g' : 'u')
#define _ACL_VALID(x) (x & ACL_VALID)
#define _ACL_READ(x) (x & ACL_READ ? 'r' : '-')
#define _ACL_WRITE(x) (x & ACL_WRITE ? 'w' : '-')
#define _ACL_EXEC(x) (x & ACL_EXECUTE ? 'x' : '-')
#define DESTROY_OBJ(object,size) { int x; for(x = 0; x < size; x++) \
                                          (void) free(object[x]); }
/* missing from errno */
#ifdef LINUX
#define ENOPOSIT 122
#endif


acl_entry_t acl_c[MAX_ACLD];
acl_s acl[MAX_ACLD];
int entry_c = MAX_ACLD;
 

int 
acl_add_perm (acl_permset_t *permset_d, const acl_perm_t perm_d)
{
  /* make sure that perm_d and permset_d are legal */
  if ( ILLEGAL_PERMS(perm_d) || ILLEGAL_PERMS(*permset_d) )
  {
    errno = EINVAL;
    return -1;
  }
  else
  {
    *permset_d |= perm_d;
    return 0;
  }
}



int 
acl_clear_perm (acl_permset_t *permset_d)
{
  bzero(permset_d, sizeof(acl_permset_t));
  return 0;
}



int 
acl_copy_entry (const acl_entry_t src_d, acl_entry_t dest_d)
{
	if ( (src_d == (acl_entry_t) NULL) || 
	    (dest_d == (acl_entry_t) NULL) )
	{
		errno = EINVAL;
		return -1;
	}

	bcopy(src_d,dest_d,sizeof(acl_s));

	return(0);
}


ssize_t 
acl_copy_ext (void *buf_p, acl_t acl_d, ssize_t size)
{
  acl_entry_t entry, *acl = (acl_t) buf_p;
  int x,acl_size = 0;


  if (acl_valid(acl_d,&entry) || (size <= 0))
  {
    errno = EINVAL;
    return ((ssize_t) -1);
  }

  for (x = 0; x < MAX_ACLD && _ACL_VALID(acl_d[x]->mode); x++ )
  {
    acl_size += sizeof (acl_s);
    if ( acl_size > size)
    {
      errno = ERANGE;
      return ((ssize_t) -1);
    }
    else
    {
      acl[x]->mode = acl_d[x]->mode;
      acl[x]->ugid = acl_d[x]->ugid;
    }
  }
    
  while ( x < MAX_ACLD )
    acl[x++] = NULL;

  return ((ssize_t) acl_size);
}



acl_t 
acl_copy_int (const void *buf_p)
{
  errno = ENOSYS;
  return (acl_t) NULL;
}


acl_entry_t 
acl_create_entry (acl_t *acl_p)
{
  int x;

  for (x = 0; x < MAX_ACLD; x++)
    if (acl_p[x] == NULL)
    {
      if ( (acl_p[x] = (acl_t) malloc(sizeof(acl_s))) == NULL)
      {
        errno = ENOMEM;
        return((acl_entry_t) NULL);
      }
      else return((acl_entry_t) acl_p[x]);
    }
    
    errno = ENOMEM;
    return((acl_entry_t) NULL);      
}


int 
acl_delete_def_fd (const int fd)
{
  errno = ENOSYS;
  return -1;
}


int 
acl_delete_def_file (const char *path_p)
{
  errno = ENOSYS;
  return -1;
}


int 
acl_delete_entry (acl_entry_t entry_d)
{
  int x;
  extern acl_s acl[MAX_ACLD];
  extern int entry_c;

  if (entry_d == (acl_entry_t) NULL)
  {
    errno = EINVAL;
    return -1;
  }

  for (x = 0; x < MAX_ACLD; x++)
  {
    if ((acl[x].mode & ACL_VALID) 
         && (acl[x].mode == entry_d->mode)
         && (acl[x].ugid == entry_d->ugid))
    {
      acl[x].mode = acl[x].ugid = 0;

      if (entry_c == x)
        ++entry_c;

      while (++x < MAX_ACLD)
      {
        acl[x-1].mode = acl[x].mode;
        acl[x-1].ugid = acl[x].ugid;
        acl[x].mode = acl[x].ugid = 0;
      }

      return 0;
    }
  }
  errno = EINVAL;
  return -1;
}
  


int 
acl_delete_perm (acl_permset_t *permset_d, acl_perm_t perm_d)
{
  /* make sure that perm_d and permset_d are legal */
  if ( ILLEGAL_PERMS(perm_d) || ILLEGAL_PERMS(*permset_d) )
  {
    errno = EINVAL;
    return -1;
  }
  else
  {
    *permset_d ^= perm_d;
    return 0;
  }
}



acl_t 
acl_dup (const acl_t acl_d)
{
  errno = ENOSYS;
  return ( (acl_t) NULL );
}


int 
acl_first_entry (acl_t acl_d)
{
  int x;
  acl_entry_t entry;
  extern acl_s acl[MAX_ACLD];
  extern int entry_c;


  if (acl_valid(acl_d, &entry))
    return -1;

  for (x = 0; x < MAX_ACLD; x++)
  {
    if ((acl_d[x] != NULL) && (acl_d[x]->mode & ACL_VALID))
    {
      acl[x].ugid = acl_d[x]->ugid;
      acl[x].mode = acl_d[x]->mode;
    }
    else
      acl[x].ugid = acl[x].mode = 0;
  }

  entry_c = 0;  

  return 0;
}


int 
acl_free (acl_t acl_d)
{
  errno = ENOSYS;
  return -1;
}



int 
acl_free_qualifier (const void *tag_qualifier_p,const acl_tag_t tag_type_d)
{
  switch(tag_type_d)
  {
    case ACL_USER:
      free((uid_t *) tag_qualifier_p);
      return 0;


    case ACL_GROUP:
      free ((gid_t *) tag_qualifier_p);
      return 0;


    case ACL_USER_OBJ:
      if (tag_qualifier_p == NULL)
        return 0;
      else
      {
        errno = EINVAL;
        return -1;
      }


    case ACL_GROUP_OBJ:
      if (tag_qualifier_p == NULL)
        return 0;
      else
      {
        errno = EINVAL;
        return -1;
      }


    case ACL_OTHER_OBJ:
      if (tag_qualifier_p == NULL)
        return 0;
      else
      {
        errno = EINVAL;
        return -1;
      }


    default:
      errno = EINVAL;
      return -1;
  }
}



int 
acl_free_text (const char *buf_p)
{
  if (buf_p == NULL)
  {
    errno = EINVAL;
    return -1;
  }
  else
    free((char *) buf_p);
  
  return 0;
}


acl_t
acl_from_text (const char *buf_p)
{
  int x,y,i,index = 0;
  int perm_val[3] = { ACL_READ, ACL_WRITE, ACL_EXECUTE };
  char tag, t[LINE_MAX], *perms;
  extern acl_entry_t acl_c[MAX_ACLD];
  extern acl_s acl[MAX_ACLD];
  extern int entry_c;
  struct passwd *pw;
  struct group *gr;

  perms = "rwx";


  for (entry_c = MAX_ACLD, index = 0, x = 0;  x < strlen(buf_p); 
       x++, index++, acl[index].mode = 0, acl[index].ugid = 0)
  {
      acl[index].mode |= ACL_VALID;
      acl_c[index] = &acl[index];
    
    for (y = 0; y < 2; y++)
    {
      for (i = 0; x < strlen(buf_p) && buf_p[x] != ':'; x++,i++)
      {
        if (buf_p[x] == ' ')
        {
          i--;
          continue;
        }
        else
          t[i] = buf_p[x];
      }

      t[i] = '\0';
      x++;

      switch(y)
      {
	case 0:
          if ((t[0] != 'u' && t[0] != 'g' && t[0] != 'o') ||
              ( strlen(t) != 1))
          {
#ifdef VERBOSE_ERR
            fprintf(stderr,"acl_from_text(): parse error -- expected [ugo]\n");
#endif
            errno = EINVAL;
            return ((acl_t) NULL);
          }
          else
            tag = t[0];
	  break;

	case 1:
          if (strlen(t) == 0)
            acl[index].ugid = 0;
          else
          {
	    if (isalpha(t[0]))
            {
              switch(tag)
              {
                case 'g':
                  if (((gr = getgrnam(t)) == NULL) 
                       || (gr->gr_gid == 0))
                  {
                    errno = EINVAL;
                    return ((acl_t) NULL);
                  }
                  else 
	            acl[index].ugid = gr->gr_gid; 
                  break;

               case 'u':
                 if (((pw = getpwnam(t)) == NULL)
                      || (pw->pw_uid == 0))
                 {
                    errno = EINVAL;
                    return ((acl_t) NULL);
                 }
                 else
                   acl[index].ugid = pw->pw_uid;
                 break;
              }
            }
            else 
              if (atoi(t) <= 0)
              {
                errno = EINVAL;
                return ((acl_t) NULL);
              }
              else 
                acl[index].ugid = atoi(t);
          }
	  break;

	default:
          errno = EINVAL;
          return ((acl_t) NULL);
      }
    }

    for (i = 0; x < strlen(buf_p) && buf_p[x] != ','; x++,i++)
    {
        if (buf_p[x] == ' ')
          continue;

	if ((buf_p[x] != perms[i] && buf_p[x] != '-') || (i >= 3))
	{
#ifdef VERBOSE_ERR
	  fprintf(stderr,"acl_from_text(): parse error -- expected [%c-]\n",perms[i]);
#endif 
          errno = EINVAL;
          return ((acl_t) NULL);
	}
	else
        {
          if (buf_p[x] != '-') 
            acl[index].mode |= perm_val[i];
        }
    }
                                                                                
    switch(tag)
    {
      case 'u':
        if (acl[index].ugid)
          acl[index].mode |= ACL_USER;
        else
          acl[index].mode |= ACL_USER_OBJ;
        break;

      case 'g':
        if (acl[index].ugid)
          acl[index].mode |= ACL_GROUP;
        else
          acl[index].mode |= ACL_GROUP_OBJ;
        break;

      case 'o':
        if (acl[index].ugid)
        {
          errno = EINVAL;
          return ((acl_t) NULL);
        }
        else
          acl[index].mode |= ACL_OTHER_OBJ;
        break;
    }
  }

  for (x = index; x < MAX_ACLD; x++)
    acl_c[x] = (acl_entry_t) NULL;

  entry_c = 0;

  return ((acl_t) acl_c);
}
     


acl_entry_t 
acl_get_entry (const acl_t acl_d)
{
  extern int entry_c;
  /* extern acl_entry_t acl_c[MAX_ACLD]; */
  extern acl_s acl[MAX_ACLD];


  if (acl_d == (acl_t) NULL)
  {
    errno = EINVAL;
    return (acl_entry_t) NULL;
  }

  if (entry_c >= MAX_ACLD)
  {
    errno = ENOPOSIT;
    return(acl_entry_t) NULL;
  }

  if (acl_c[entry_c] == NULL)
  {
    entry_c = MAX_ACLD;
    errno = ENOPOSIT;
    return (acl_entry_t) NULL;
  }

  return (acl_entry_t) &acl[entry_c++];
}



acl_t 
acl_get_fd (const int fd, const acl_type_t type_d)
{
  errno = ENOSYS;
  return ( (acl_t) NULL );
}



acl_t 
acl_get_file (const char *path_p, const acl_type_t type_d)
{
  extern acl_entry_t acl_c[MAX_ACLD];
  extern acl_s acl[MAX_ACLD];
  struct stat buf_s;
  int x;


  if ((type_d != ACL_TYPE_ACCESS) && (type_d != ACL_TYPE_DEFAULT))
  {
    errno = EINVAL;
    return((acl_t) NULL);
  }

  /* not supporting optional default ACL */
  if (type_d == ACL_TYPE_DEFAULT)
  {
    errno = ENOSYS;
    return((acl_t) NULL);
  }

  /* check perms */
  if (stat(path_p,&buf_s) < 0)
    return((acl_t) NULL);

  /* read the acl */ 
  if ( (x = readacl(path_p,&acl)) != 0)
  {
    errno = ( x * -1 );
    return((acl_t) NULL);
  }
  else
  {
    for (x = 0; x < MAX_ACLD; x++)
    {
      if (_ACL_VALID(acl[x].mode))
        acl_c[x] = (acl_entry_t) &acl[x];
      else
        acl_c[x] = (acl_entry_t) NULL;
    }
    /* reset the current entry */
    entry_c = 0;

    return((acl_t) acl_c);
  }
}



int
acl_get_permset(const acl_entry_t entry_d, acl_permset_t *permset_p)
{
  if (entry_d == NULL)
  {
    errno = EINVAL;
    return -1;
  }

  *permset_p = (entry_d->mode | (ACL_READ | ACL_WRITE | ACL_EXECUTE));
  
  return 0;
}


void 
*acl_get_qualifier (const acl_entry_t entry_d)
{
  gid_t *g;
  uid_t *u;

  if (entry_d == NULL)
  {
    errno = EINVAL;
    return NULL;
  }

  switch(entry_d->mode)
  {
    case ACL_USER:
      if ( (g = malloc(sizeof(gid_t))) == NULL)
      {
        errno = ENOMEM;
        return NULL;
      }
      *g = (gid_t) entry_d->ugid;
      return ((void *) g);


    case ACL_GROUP:
      if ( (u = malloc(sizeof(uid_t))) == NULL)
      {
        errno = ENOMEM;
        return NULL;
      }
      *u = (uid_t) entry_d->ugid;
      return ((void *) u);


    default:
      errno = EINVAL;
      return NULL;
  }
}


int 
acl_get_tag_type (const acl_entry_t entry_d, acl_tag_t *tag_type_p)
{
  errno = ENOSYS;
  return -1;
}


acl_t 
acl_init (const size_t size)
{
  errno = ENOSYS;
  return ( (acl_t) NULL );
}


int 
acl_set_fd (const int fd, const acl_type_t type_d, const acl_t acl_d)
{
  errno = ENOSYS;
  return -1;
}



int 
acl_set_file (const char *path_p, const acl_type_t type_d,const acl_t acl_d)
{
  acl_s acl[MAX_ACLD];
  acl_entry_t entry;
  int x,y;

  if ((type_d != ACL_TYPE_ACCESS) && (type_d != ACL_TYPE_DEFAULT))
  {
    errno = EINVAL;
    return(-1);
  }

  /* not supporting optional default ACL */
  if (type_d == ACL_TYPE_DEFAULT)
  {
    errno = ENOSYS;
    return -1;
  }

  if (acl_valid(acl_d, &entry))
    return -1;


  /* place the acl entries into an acl_s object */
  for (y = 3, x = 0; x < MAX_ACLD; x++)
  {
    if (acl_d[x] != NULL)
    {
      /* put the object entries in their respective location */
      /* doing this here save alot of time as opposed to having */
      /* the kernel do it */
      switch (acl_d[x]->mode & (ACL_USER_OBJ | ACL_GROUP_OBJ | ACL_OTHER_OBJ))
      { 
        case ACL_USER_OBJ:
          acl[0].ugid = acl_d[x]->ugid;
          acl[0].mode = acl_d[x]->mode;
          break;

        case ACL_GROUP_OBJ:
          acl[1].ugid = acl_d[x]->ugid;
          acl[1].mode = acl_d[x]->mode;
          break;

        case ACL_OTHER_OBJ:
          acl[2].ugid = acl_d[x]->ugid;
          acl[2].mode = acl_d[x]->mode;
          break;

        default:
          acl[y].ugid = acl_d[x]->ugid;
          acl[y].mode = acl_d[x]->mode;
          y++;
          break;
      }
    }
    else
      acl[x].mode = acl[x].ugid = 0;
  }
  
  if ( (x = writeacl(path_p,acl)) < 0)
  {
    errno = ( x * -1);
    return -1;
  }
  else return 0;
}



int 
acl_set_permset (acl_entry_t entry_d, const acl_permset_t permset_d)
{
  if ((entry_d == NULL) ||
      (ILLEGAL_PERMS(permset_d)))
  {
    errno = EINVAL;
    return -1;
  }

  entry_d->mode |= permset_d;
  
  return 0;
}




int 
acl_set_qualifier (acl_entry_t entry_d, const void *tag_qualifier_p)
{
  switch(entry_d->mode)
  {
    case ACL_USER:
      entry_d->ugid = (int) (u_short *) tag_qualifier_p;
      break;

    case ACL_GROUP:
      entry_d->ugid = (int) (u_short *) tag_qualifier_p;
      break;

    default:
      errno = EINVAL;
      return(-1);
  }
}




int 
acl_set_tag_type (acl_entry_t entry_d, const acl_tag_t tag_type_d)
{
  if (entry_d == (acl_entry_t) NULL)
  {
    errno = EINVAL;
    return (-1);
  }

  switch(tag_type_d)
  {
    case ACL_USER_OBJ:    
      entry_d->mode |= ACL_USER_OBJ;
      break;

    case ACL_OTHER_OBJ:
      entry_d->mode |= ACL_OTHER_OBJ;
      break;

    case ACL_GROUP_OBJ: 
      entry_d->mode |= ACL_GROUP_OBJ;
      break;

    case ACL_USER:
      entry_d->mode |= ACL_USER;
      break;

    case ACL_GROUP:
      entry_d->mode |= ACL_GROUP;
      break;

    default:
      errno = EINVAL;
      return -1;
  }
  return(0);
}




ssize_t 
acl_size (const acl_t acl_d)
{
  int x;
  ssize_t t=0;
  acl_t acl = (acl_t) &acl_d;

  if (acl == (acl_t) NULL)
  {
    errno = EINVAL;
    return ( (ssize_t) - 1 );    
  }

  for (x = 0; x < MAX_ACLD; x++)
    if (acl[x] != NULL)
      t += (ssize_t) sizeof(acl_s); 

  return(t);
}




char 
*acl_to_text (const acl_t acl_d, ssize_t *len_p)
{
  char buf_s[LINE_MAX], buf_e[LINE_MAX], ugid[10], *buf_r, *strdup(),*tag_type;
  acl_entry_t entry;
  int x=0;

 
  buf_s[0] = '\0';

  if (acl_valid(acl_d,&entry))
  {
    return (NULL);
  }

  while ((acl_d[x] != NULL) && 
         ( x < MAX_ACLD) && 
         (_ACL_VALID(acl_d[x]->mode)))
  {

    switch(acl_d[x]->mode & (ACL_GROUP | ACL_USER | ACL_OTHER_OBJ |
                           ACL_GROUP_OBJ | ACL_USER_OBJ)) 
    {
      case ACL_GROUP:
        tag_type = "g";
        break;
    
      case ACL_USER:
        tag_type = "u";
        break;

      case ACL_OTHER_OBJ:
        tag_type = "o";
        break;

      case ACL_GROUP_OBJ:
        tag_type = "g";
        break;

      case ACL_USER_OBJ:
        tag_type = "u";
        break;
    }   

    if (acl_d[x]->ugid)
      sprintf(ugid,"%u",acl_d[x]->ugid);
    else
      sprintf(ugid,"");
 

    if (x == 0)
      sprintf(buf_e,"%s:%s:%c%c%c",tag_type, ugid,
                                   _ACL_READ(acl_d[x]->mode),
                                   _ACL_WRITE(acl_d[x]->mode),
	                           _ACL_EXEC(acl_d[x]->mode));
    else
      sprintf(buf_e," , %s:%s:%c%c%c",tag_type, ugid,
                                      _ACL_READ(acl_d[x]->mode),
                                      _ACL_WRITE(acl_d[x]->mode),
	                              _ACL_EXEC(acl_d[x]->mode));
 

    strcat(buf_s,buf_e);
    x++;
  }
  
  *len_p = strlen(buf_s);

  if ( (buf_r = strdup(buf_s)) == NULL)
  {
    errno = ENOMEM;
    return((char *) NULL);
  }
  else
    return (buf_r);
}



int 
acl_valid (acl_t acl_d, acl_entry_t *entry_p)
{
  int x, y, user = MAX_ACLD, group = MAX_ACLD, other = MAX_ACLD;
  int groups[MAX_ACLD], users[MAX_ACLD];
  int gi = 0, ui = 0;
 

  if (acl_d == (acl_t) NULL)
  {
    entry_p = (acl_entry_t *) NULL;
    errno = EINVAL;
    return -1;
  }

  for (x = 0; x < MAX_ACLD; x++)
  {
    if (acl_d[x] != NULL && acl_d[x]->mode & ACL_VALID)
    {  
      switch(acl_d[x]->mode & (ACL_USER_OBJ | ACL_GROUP_OBJ | ACL_OTHER_OBJ))
      {
        case ACL_USER_OBJ:
          if ((user != MAX_ACLD) || (acl_d[x]->ugid != 0) 
               || (acl_d[x]->mode & ACL_USER & ACL_GROUP)) 
          {
            *entry_p = (acl_entry_t) &acl_d[x];
            errno = EINVAL;
#ifdef VERBOSE_ERR
            fprintf(stderr,"multiple user object entries\n");
#endif
            return -1;
          }
          else
            user = x;

          break;


        case ACL_GROUP_OBJ:
          if ((group != MAX_ACLD) || (acl_d[x]->ugid != 0)
              || (acl_d[x]->mode & ACL_USER & ACL_GROUP))
          {
            *entry_p = (acl_entry_t) &acl_d[x];
            errno = EINVAL;
#ifdef VERBOSE_ERR
            fprintf(stderr,"duplicate group object entries\n");
#endif
            return -1;
          }
          else
            group = x;

          break;

        case ACL_OTHER_OBJ:
          if ((other != MAX_ACLD) || (acl_d[x]->ugid != 0) 
              || (acl_d[x]->mode & ACL_USER & ACL_GROUP))
          {
            *entry_p = (acl_entry_t) &acl_d[x];
            errno = EINVAL;
#ifdef VERBOSE_ERR
            fprintf(stderr,"acl_valid(): duplicate other object entries\n");
#endif
            return -1;
          }
          else
            other = x;
          
          break;

        default:
          /* check for multiple tag types */
          if ((acl_d[x]->mode & ACL_USER) && (acl_d[x]->mode & ACL_GROUP))
          {
             *entry_p = (acl_entry_t) &acl[x];
             errno = EINVAL;
#ifdef VERBOSE_ERR
             fprintf(stderr,"multiple tag types\n");
#endif
             return -1;
          } 
          else
          {
            if (acl_d[x]->mode & ACL_GROUP)
            {
              for (y = 0; y < gi; y++)
                if (groups[y] == acl_d[x]->ugid)
                {
                  entry_p = &acl_d[x];
                  errno = EINVAL;
#ifdef VERBOSE_ERR
                  fprintf(stderr,"duplicate group entries\n");
#endif
                  return -1;
                }
              groups[gi++] = acl_d[x]->ugid; 
            }  
            else
            {
              for (y = 0; y < ui; y++)
                if (users[y] == acl_d[x]->ugid)
                {
                  entry_p = &acl_d[x];
                  errno = EINVAL;
#ifdef VERBOSE_ERR
                  fprintf(stderr,"acl_valid(): duplicate user entries\n");
#endif
                  return -1;
                }
              users[ui++] = acl_d[x]->ugid; 
            }  
          }
          break;
      }
    }
  }
  /* ..and check for missing object entries */
  if ((user == MAX_ACLD) || (group == MAX_ACLD) || (other == MAX_ACLD)) 
  {
    entry_p = (acl_entry_t *) NULL;
    errno = EINVAL;
#ifdef VERBOSE_ERR
    fprintf(stderr,"missing object entries\n");
#endif
    return -1;
  }
  return 0;
}


